Method: RLTK::CG::SimpleAggregateType#initialize

Defined in:
lib/rltk/cg/type.rb

#initialize(overloaded, size_or_address_space = 0) ⇒ SimpleAggregateType

Used to initialize ArrayTypes and PointerTypes.

Parameters:

  • overloaded (FFI::Pointer, Type)

    Pointer to an existing aggregate type or a Type object that describes the objects that will be stored in an aggregate type.



239
240
241
242
243
244
245
246
247
248
249
250
# File 'lib/rltk/cg/type.rb', line 239

def initialize(overloaded, size_or_address_space = 0)
	@ptr =
	case overloaded
	when FFI::Pointer
		overloaded
	else
		@element_type	= check_cg_type(overloaded, Type, 'overloaded')
		bname		= Bindings.get_bname(self.class.short_name)
		
		Bindings.send(bname, @element_type, size_or_address_space)
	end
end