Exception: RLTK::HandledError

Inherits:
StandardError
  • Object
show all
Defined in:
lib/rltk/parser.rb

Overview

An error of this type is raised when the parser encountered a error that was handled by an error production.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(errors, result) ⇒ HandledError

Instantiate a new HandledError object with errors.



54
55
56
57
# File 'lib/rltk/parser.rb', line 54

def initialize(errors, result)
  @errors = errors
  @result = result
end

Instance Attribute Details

#errorsArray<Object> (readonly)

The errors as reported by the parser.



45
46
47
# File 'lib/rltk/parser.rb', line 45

def errors
  @errors
end

#resultObject (readonly)

The result that would have been returned by the call to parse.



48
49
50
# File 'lib/rltk/parser.rb', line 48

def result
  @result
end