Class: RLTK::Parser::Action
Overview
The Action class is used to indicate what action the parser should take given a current state and input token.
Instance Attribute Summary collapse
-
#id ⇒ Integer
readonly
ID of this action.
Instance Method Summary collapse
-
#initialize(id = nil) ⇒ Action
constructor
A new instance of Action.
Constructor Details
#initialize(id = nil) ⇒ Action
Returns a new instance of Action.
1545 1546 1547 |
# File 'lib/rltk/parser.rb', line 1545 def initialize(id = nil) @id = id end |
Instance Attribute Details
#id ⇒ Integer (readonly)
Returns ID of this action.
1542 1543 1544 |
# File 'lib/rltk/parser.rb', line 1542 def id @id end |