Method: RLTK::Parser::ParseStack#position

Defined in:
lib/rltk/parser.rb

#positionStreamPosition

Returns Position data for the last symbol on the stack.

Returns:

  • (StreamPosition)

    Position data for the last symbol on the stack.



1316
1317
1318
1319
1320
1321
1322
# File 'lib/rltk/parser.rb', line 1316

def position
	if @positions.empty?
		StreamPosition.new
	else
		@positions.last.clone
	end
end