Class: RLTK::Parser::ParseStack
Overview
The ParseStack class is used by a Parser to keep track of state during parsing.
Instance Attribute Summary collapse
-
#id ⇒ Integer
readonly
ID of this parse stack.
-
#output_stack ⇒ Array<Object>
readonly
Array of objects produced by Reduce actions.
-
#state_stack ⇒ Array<Integer>
readonly
Array of states used when performing Reduce actions.
Instance Method Summary collapse
-
#branch(new_id) ⇒ ParseStack
Branch this stack, effectively creating a new copy of its internal state.
-
#initialize(id, ostack = [], sstack = [0], nstack = [], connections = [], labels = [], positions = []) ⇒ ParseStack
constructor
Instantiate a new ParserStack object.
-
#pop(n = 1) ⇒ Array<Array<Object, StreamPosition>>
Pop some number of objects off of the inside stacks.
-
#position ⇒ StreamPosition
Position data for the last symbol on the stack.
-
#push(state, o, node0, position) ⇒ void
Push new state and other information onto the stack.
-
#result ⇒ Object
Fetch the result stored in this ParseStack.
-
#state ⇒ Integer
Current state of this ParseStack.
-
#tree ⇒ String
Representation of the parse tree in the DOT langauge.
Constructor Details
#initialize(id, ostack = [], sstack = [0], nstack = [], connections = [], labels = [], positions = []) ⇒ ParseStack
Instantiate a new ParserStack object.
1292 1293 1294 1295 1296 1297 1298 1299 1300 1301 1302 |
# File 'lib/rltk/parser.rb', line 1292 def initialize(id, ostack = [], sstack = [0], nstack = [], connections = [], labels = [], positions = []) @id = id @node_stack = nstack @output_stack = ostack @state_stack = sstack @connections = connections @labels = labels @positions = positions end |
Instance Attribute Details
#id ⇒ Integer (readonly)
Returns ID of this parse stack.
1275 1276 1277 |
# File 'lib/rltk/parser.rb', line 1275 def id @id end |
Instance Method Details
#branch(new_id) ⇒ ParseStack
Branch this stack, effectively creating a new copy of its internal state.
1310 1311 1312 1313 |
# File 'lib/rltk/parser.rb', line 1310 def branch(new_id) ParseStack.new(new_id, @output_stack.clone, @state_stack.clone, @node_stack.clone, @connections.clone, @labels.clone, @positions.clone) end |
#pop(n = 1) ⇒ Array<Array<Object, StreamPosition>>
Pop some number of objects off of the inside stacks.
1351 1352 1353 1354 1355 1356 1357 1358 1359 1360 |
# File 'lib/rltk/parser.rb', line 1351 def pop(n = 1) @state_stack.pop(n) # Pop the node stack so that the proper edges can be added # when the production's left-hand side non-terminal is # pushed onto the stack. @cbuffer = @node_stack.pop(n) [@output_stack.pop(n), @positions.pop(n)] end |
#position ⇒ StreamPosition
Returns Position data for the last symbol on the stack.
1316 1317 1318 1319 1320 1321 1322 |
# File 'lib/rltk/parser.rb', line 1316 def position if @positions.empty? StreamPosition.new else @positions.last.clone end end |
#push(state, o, node0, position) ⇒ void
This method returns an undefined value.
Push new state and other information onto the stack.
1332 1333 1334 1335 1336 1337 1338 1339 1340 1341 1342 1343 1344 |
# File 'lib/rltk/parser.rb', line 1332 def push(state, o, node0, position) @state_stack << state @output_stack << o @node_stack << @labels.length @labels << if CFG::is_terminal?(node0) and o then node0.to_s + "(#{o})" else node0 end @positions << position if CFG::is_nonterminal?(node0) @cbuffer.each do |node1| @connections << [@labels.length - 1, node1] end end end |
#result ⇒ Object
Fetch the result stored in this ParseStack. If there is more than one object left on the output stack there is an error.
1366 1367 1368 1369 1370 1371 1372 |
# File 'lib/rltk/parser.rb', line 1366 def result if @output_stack.length == 1 return @output_stack.last else raise InternalParserException, "The parsing stack should have 1 element on the output stack, not #{@output_stack.length}." end end |
#state ⇒ Integer
Returns Current state of this ParseStack.
1375 1376 1377 |
# File 'lib/rltk/parser.rb', line 1375 def state @state_stack.last end |
#tree ⇒ String
Returns Representation of the parse tree in the DOT langauge.
1380 1381 1382 1383 1384 1385 1386 1387 1388 1389 1390 1391 1392 1393 1394 1395 1396 1397 1398 1399 1400 |
# File 'lib/rltk/parser.rb', line 1380 def tree tree = "digraph tree#{@id} {\n" @labels.each_with_index do |label, i| tree += "\tnode#{i} [label=\"#{label}\"" if CFG::is_terminal?(label) tree += " shape=box" end tree += "];\n" end tree += "\n" @connections.each do |from, to| tree += "\tnode#{from} -> node#{to};\n" end tree += "}" end |